5.2 Iterazioni e anelli

# Iterazioni e anelli

Fino a questo momento, abbiamo passato molto tempo a spiegare i differenti suoni che puoi ottenere utilizzando le funzioni 'play' e 'sample'. Hai anche imparato come scansionare il tempo tra i suoni attraverso la funzione 'sleep'.

Come avrai probabilmente scoperto, puoi divertirti un sacco con questi blocchi basilari. Tuttavia, quando comincerai a utilizzare il potere del codice per strutturare le tue composizioni e la tua musica, tutta una nuova dimensione di divertimento ti si aprirà davanti. Nelle prossime sezioni, illustreremo alcuni di questi potenti strumenti. I primi saranno le iterazioni e gli anelli.

## Ripetizione

Hai mai scritto una stringa di codice che ti piacerebbe ripetere alcune volte? Per esempio, qualcosa tipo:

```
play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25
```

Che cosa dobbiamo fare se vogliamo ripetere questo codice tre volte? Bene, potremmo semplicemente copiarlo e incollarlo per tre volte di seguito:

```
play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25
play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25
play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25
```

Certo che è proprio un codice lunghissimo! Che cosa accadrebbe se tu volessi cambiare il campione indicato con 'elec_plip'? Dovresti trovare tutti i punti in cui è menzionato l'originale, cioè 'elec_blup', e cambiarlo. Ma ancora peggio, che cosa accadrebbe se tu volessi ripetere il codice originale per 50 o adirittura 1000 volte? A questo punto avresti un codice enorme, con un sacco di linee da variare se tu volessi effettuare un qualsiasi cambiamento.

## Iterazione

Ripetere più volte una o più stringhe di codice può essere semplice come dire *fai questo tre volte*. Alla fine, è proprio questo che faremo. Ti ricordi del nostro vecchio amico chiamato blocco di codice? Possiamo utilizzarlo per indicare l'inizio e la fine di un gruppo di codice che vogliamo ripetere per tre volte. Quindi, invece di scrivere *fai questo tre volte*, noi scriveremo '3.times do'. Non è troppo difficile, vero? Devi solo ricordare di scrivere 'end' alla fine del codice che vuoi ripetere:

```
3.times do
  play 50
  sleep 0.5
  sample :elec_blup
  sleep 0.5
  play 62
  sleep 0.25
end
```

Non è molto più facile che fare copia e incolla? Possiamo usare questa struttura per creare un sacco di strutture interessanti fatte con la ripetizione:

```
4.times do
  play 50
  sleep 0.5
end
8.times do
  play 55, release: 0.2
  sleep 0.25
end
4.times do
  play 50
  sleep 0.5
end
```

## Intrecciare le iterazioni

Possiamo anche mettere le iterazioni una dentro l'altra per creare delle strutture interessanti. Per esempio:

```
4.times do
  sample :drum_heavy_kick
  2.times do
    sample :elec_blip2, rate: 2
    sleep 0.25
  end
  sample :elec_snare
  4.times do
    sample :drum_tom_mid_soft
    sleep 0.125
  end
end
```

## Creare anelli (loop)

Se volessi che un blocco di codice si ripetesse tante volte, potresti trovare utile l'utilizzo di numeri grandi come '1000.times do'. In questi casi, però, faresti forse meglio a dire a Sonic Pi di ripetere il blocco all'infinito (o almeno finché non premi stop!). Proviamo a creare un anello con il campione amen_break:

```
loop do
  sample :loop_amen
  sleep sample_duration :loop_amen
end
```

Un'aspetto importante da sapere degli anelli è che essi si comportano come dei buchi neri per il codice. Infatti, una volta che il codice entra in un anello, non riesce più a uscirne finché non premi stop. Il codice continuerà a girare in cerchio all'infinito. Quindi, se hai scritto del codice successivamente a un anello, questo non verrà mai eseguito. Nel caso seguente il piatto scritto dopo l'anello non suonerà mai:

```
loop do
  play 50
  sleep 1
end
sample :drum_cymbal_open
```

Adesso puoi cominciare a strutturare il tuo codice inserendo le iterazioni e gli anelli, vai!
